/*
 * Rename.cs --
 *
 * Copyright (c) 2007-2012 by Joe Mistachkin.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * RCS: @(#) $Id: $
 */

using Eagle._Attributes;
using Eagle._Components.Private;
using Eagle._Components.Public;
using Eagle._Constants;
using Eagle._Containers.Public;
using Eagle._Interfaces.Public;

namespace Eagle._Commands
{
    [ObjectId("fe410cf6-1f44-47d5-a9dc-613770302383")]
    [CommandFlags(CommandFlags.Safe | CommandFlags.Standard)]
    [ObjectGroup("procedure")]
    internal sealed class Rename : Core
    {
        public Rename(
            ICommandData commandData
            )
            : base(commandData)
        {
            // do nothing.
        }

        #region IExecute Members
        public override ReturnCode Execute(
            Interpreter interpreter,
            IClientData clientData,
            ArgumentList arguments,
            ref Result result
            )
        {
            ReturnCode code = ReturnCode.Ok;

            if (interpreter != null)
            {
                if (arguments != null)
                {
                    if (arguments.Count >= 3)
                    {
                        OptionDictionary options = new OptionDictionary(
                            new Option[] { 
                            new Option(null, OptionFlags.None, Index.Invalid, Index.Invalid, "-nodelete", null),
                            new Option(typeof(IdentifierKind), OptionFlags.MustHaveEnumValue, Index.Invalid, Index.Invalid, "-kind",
                                new Variant(IdentifierKind.None)),
                            new Option(null, OptionFlags.None, Index.Invalid, Index.Invalid, Option.EndOfOptions, null)
                        });

                        int argumentIndex = Index.Invalid;

                        code = interpreter.GetOptions(options, arguments, 0, 1, Index.Invalid, false, ref argumentIndex, ref result);

                        if (code == ReturnCode.Ok)
                        {
                            if ((argumentIndex != Index.Invalid) && 
                                ((argumentIndex + 1) < arguments.Count) && 
                                ((argumentIndex + 2) >= arguments.Count))
                            {
                                Variant value = null;
                                IdentifierKind kind = IdentifierKind.None;

                                if (options.IsPresent("-kind", ref value))
                                    kind = (IdentifierKind)value.Value;

                                bool delete = true;

                                if (options.IsPresent("-nodelete"))
                                    delete = false;

                                //
                                // TODO: Modify this when full [namespace] support has been added.
                                //
                                string oldName = StringOps.TrimLeadingColons(arguments[argumentIndex]);
                                string newName = StringOps.TrimLeadingColons(arguments[argumentIndex + 1]);

                                switch (kind)
                                {
                                    case IdentifierKind.None:
                                        {
                                            if (interpreter.DoesIExecuteExist(oldName) == ReturnCode.Ok)
                                                code = interpreter.RenameIExecute(
                                                    oldName, newName, delete, ref result);
                                            if (interpreter.DoesProcedureExist(oldName) == ReturnCode.Ok)
                                                code = interpreter.RenameProcedure(
                                                    oldName, newName, delete, ref result);
                                            else
                                                code = interpreter.RenameCommand(
                                                    oldName, newName, delete, ref result);
                                            break;
                                        }
                                    case IdentifierKind.Command:
                                        {
                                            code = interpreter.RenameCommand(
                                                oldName, newName, delete, ref result);
                                            break;
                                        }
                                    case IdentifierKind.Procedure:
                                        {
                                            code = interpreter.RenameProcedure(
                                                oldName, newName, delete, ref result);
                                            break;
                                        }
                                    case IdentifierKind.IExecute:
                                        {
                                            code = interpreter.RenameIExecute(
                                                oldName, newName, delete, ref result);
                                            break;
                                        }
                                    default:
                                        {
                                            result = "identifier kind not supported";
                                            code = ReturnCode.Error;
                                            break;
                                        }
                                }
                            }
                            else
                            {
                                if ((argumentIndex != Index.Invalid) &&
                                    Option.LooksLikeOption(arguments[argumentIndex]))
                                {
                                    result = OptionDictionary.BadOption(options, arguments[argumentIndex]);
                                }
                                else
                                {
                                    result = "wrong # args: should be \"rename ?options? oldName newName\"";
                                }

                                code = ReturnCode.Error;
                            }
                        }
                    }
                    else
                    {
                        result = "wrong # args: should be \"rename ?options? oldName newName\"";
                        code = ReturnCode.Error;
                    }
                }
                else
                {
                    result = "invalid argument list";
                    code = ReturnCode.Error;
                }
            }
            else
            {
                result = "invalid interpreter";
                code = ReturnCode.Error;
            }

            return code;
        }
        #endregion
    }
}
