/*
 * Time.cs --
 *
 * Copyright (c) 2007-2012 by Joe Mistachkin.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * RCS: @(#) $Id: $
 */

using Eagle._Attributes;
using Eagle._Components.Private;
using Eagle._Components.Public;
using Eagle._Constants;
using Eagle._Containers.Public;
using Eagle._Interfaces.Public;

namespace Eagle._Commands
{
    [ObjectId("3921a1d3-c345-42e5-a567-b9ca2ef6b366")]
#if NATIVE
    [CommandFlags(CommandFlags.NativeCode | CommandFlags.Safe | CommandFlags.Standard)]
#else
    [CommandFlags(CommandFlags.Safe | CommandFlags.Standard)]
#endif
    [ObjectGroup("time")]
    internal sealed class Time : Core
    {
        public Time(
            ICommandData commandData
            )
            : base(commandData)
        {
            // do nothing.
        }

        #region IExecute Members
        public override ReturnCode Execute(
            Interpreter interpreter,
            IClientData clientData,
            ArgumentList arguments,
            ref Result result
            )
        {
            ReturnCode code = ReturnCode.Ok;

            if (interpreter != null)
            {
                if (arguments != null)
                {
                    if ((arguments.Count == 2) || (arguments.Count == 3))
                    {
                        long iterations = 1;

                        if (arguments.Count == 3)
                        {
                            if (Value.GetWideInteger2(
                                    arguments[2], ValueFlags.AnyWideInteger,
                                    interpreter.CultureInfo, ref iterations,
                                    ref result) != ReturnCode.Ok)
                            {
                                return ReturnCode.Error;
                            }
                        }

                        long startCount = PerformanceOps.GetCount();
                        long newIterations = iterations;

                        //
                        // NOTE: If the number of iterations is negative one,
                        //       we will keep going forever (i.e. until
                        //       canceled).
                        //
                        while (true)
                        {
                            code = interpreter.EvaluateScript(
                                arguments[1], ref result);

                            if (code != ReturnCode.Ok)
                                break;

                            if (newIterations == Count.Invalid)
                                continue;

                            if (--newIterations <= 0)
                                break;
                        }

                        if (code == ReturnCode.Ok)
                        {
                            long stopCount = PerformanceOps.GetCount();

                            double microseconds;

                            if (iterations > 0)
                                microseconds = PerformanceOps.GetMicroseconds(
                                    startCount, stopCount, iterations);
                            else
                                microseconds = 0;

                            result = FormatOps.Performance(microseconds);
                        }
                    }
                    else
                    {
                        result = "wrong # args: should be \"time script ?count?\"";
                        code = ReturnCode.Error;
                    }
                }
                else
                {
                    result = "invalid argument list";
                    code = ReturnCode.Error;
                }
            }
            else
            {
                result = "invalid interpreter";
                code = ReturnCode.Error;
            }

            return code;
        }
        #endregion
    }
}
