@ECHO OFF

::
:: flight.bat --
::
:: Extensible Adaptable Generalized Logic Engine (Eagle)
:: Release Preparation & Build Tool
::
:: Copyright (c) 2007-2012 by Joe Mistachkin.  All rights reserved.
::
:: See the file "license.terms" for information on usage and redistribution of
:: this file, and for a DISCLAIMER OF ALL WARRANTIES.
::
:: RCS: @(#) $Id: $
::

SETLOCAL
ECHO STARTED ON %DATE% AT %TIME% BY %USERDOMAIN%\%USERNAME%

REM SET __ECHO=ECHO
REM SET __ECHO2=ECHO
REM SET __ECHO3=ECHO
IF NOT DEFINED _AECHO (SET _AECHO=REM)
IF NOT DEFINED _CECHO (SET _CECHO=REM)
IF NOT DEFINED _VECHO (SET _VECHO=REM)

SET APPEND=^>^>
IF DEFINED __ECHO SET APPEND=^^^>^^^>

SET PIPE=^|
IF DEFINED __ECHO SET PIPE=^^^|

%_AECHO% Running %0 %*

REM SET DFLAGS=/L

%_VECHO% DFlags = '%DFLAGS%'

SET FFLAGS=/V /F /G /H /I /R /Y /Z

%_VECHO% FFlags = '%FFLAGS%'

SET CONFIGURATION=%1

IF DEFINED CONFIGURATION (
  CALL :fn_UnquoteVariable CONFIGURATION
) ELSE (
  %_AECHO% No configuration specified, using default...
  SET CONFIGURATION=Release
)

%_VECHO% Configuration = '%CONFIGURATION%'

REM ****************************************************************************
REM ********************** Set Miscellaneous Environment ***********************
REM ****************************************************************************

IF NOT DEFINED PACKAGE_PLATFORM (
  %_AECHO% No package platform specified, using default...
  SET PACKAGE_PLATFORM=Win32
)

%_VECHO% PackagePlatform = '%PACKAGE_PLATFORM%'

IF NOT DEFINED PACKAGE_CONFIGURATION (
  %_AECHO% No package configuration specified, using default...
  SET PACKAGE_CONFIGURATION=%CONFIGURATION%Dll
)

%_VECHO% PackageConfiguration = '%PACKAGE_CONFIGURATION%'

REM
REM HACK: If we are not building the NetFx40 build configuration then Garuda
REM       will not be built; therefore, disable other steps that require it.
REM
IF DEFINED NONETFX40 (
  SET NOPACKAGE=1
)

IF NOT DEFINED HASHBASES (
  SET HASHBASES=EagleSource EagleSetup EagleBinary EagleRuntime EagleCore

  IF NOT DEFINED NOPACKAGE (
    CALL :fn_AppendVariable HASHBASES " GarudaSetup GarudaBinary"
    CALL :fn_AppendVariable HASHBASES " GarudaRuntime GarudaCore"
  )
)

%_VECHO% HashBases = '%HASHBASES%'

IF NOT DEFINED HASHEXTS (
  SET HASHEXTS=exe rar zip
)

%_VECHO% HashExts = '%HASHEXTS%'

IF NOT DEFINED PATCHLEVELFILES (
  SET PATCHLEVELFILES=PatchLevel.cs
)

%_VECHO% PatchLevelFiles = '%PATCHLEVELFILES%'

IF NOT DEFINED TAGFILES (
  SET TAGFILES=BuildInfo.cs PatchLevel.cs
)

%_VECHO% TagFiles = '%TAGFILES%'

IF NOT DEFINED PACKAGE_TAGFILES (
  SET PACKAGE_TAGFILES=pkgVersion.h
)

%_VECHO% PackageTagFiles = '%PACKAGE_TAGFILES%'

SET ARGS=%*

%_VECHO% Args = '%ARGS%'

REM ****************************************************************************
REM ***************************** Set Build Paths ******************************
REM ****************************************************************************

SET RELEASES=%~dp0\..\..\Releases
SET RELEASES=%RELEASES:\\=\%

%_VECHO% Releases = '%RELEASES%'

SET LOGDIR=%RELEASES%
SET LOGPREFIX=EagleFlight

%_VECHO% LogDir = '%LOGDIR%'
%_VECHO% LogPrefix = '%LOGPREFIX%'

SET NUGETBASEPATH=%TEMP%\Eagle_NuGet_%CONFIGURATION%
SET NUGETOUTPUTDIR=%RELEASES%\%CONFIGURATION%

%_VECHO% NuGetBasePath = '%NUGETBASEPATH%'
%_VECHO% NuGetOutputDir = '%NUGETOUTPUTDIR%'

SET TOOLS=%~dp0
SET TOOLS=%TOOLS:~0,-1%

%_VECHO% Tools = '%TOOLS%'

SET PACKAGE_TOOLS=%~dp0\..\..\Package\Tools
SET PACKAGE_TOOLS=%PACKAGE_TOOLS:\\=\%

%_VECHO% PackageTools = '%PACKAGE_TOOLS%'

SET ROOT=%~dp0\..\..
SET ROOT=%ROOT:\\=\%

%_VECHO% Root = '%ROOT%'

REM
REM NOTE: When performing the release process for just the core library, only
REM       look for files to tag inside of the "Library" sub-directory.
REM
IF DEFINED COREONLY (
  SET TAGPATTERN=\Library
) ELSE (
  CALL :fn_UnsetVariable TAGPATTERN
)

%_VECHO% TagPattern = '%TAGPATTERN%'

REM
REM NOTE: Reset the error level now so that we can easily check it against zero
REM       later.
REM
CALL :fn_ResetErrorLevel

REM ****************************************************************************
REM *********************** Verify Eagle Shell (in LKG) ************************
REM ****************************************************************************
REM
REM NOTE: Attempt to use the last known good build of Eagle to set the source
REM       identifier for the current build.  If the NOLKG environment variable
REM       is defined, we assume that the last known good build of Eagle will
REM       already be in the path; otherwise, the build will fail ^(below^) when
REM       we try to update the source identifier.
REM
IF DEFINED NOLKG GOTO skip_lastKnownGood
IF NOT DEFINED LKG GOTO skip_lastKnownGood

SET SRCLKGDIR=%LKG%\Eagle\bin
SET SRCLKGDIR=%SRCLKGDIR:\\=\%

%_VECHO% SrcLkgDir = '%SRCLKGDIR%'

IF NOT EXIST "%SRCLKGDIR%\EagleShell.exe" (
  ECHO The file "%SRCLKGDIR%\EagleShell.exe" does not exist.
  GOTO usage
)

%__ECHO% "%SRCLKGDIR%\EagleShell.exe" /? > NUL 2>&1

IF ERRORLEVEL 1 (
  ECHO The "%SRCLKGDIR%\EagleShell.exe" tool appears to be missing.
  GOTO usage
)

%_AECHO% Using LKG "EagleShell.exe" tool from "%SRCLKGDIR%"...

SET PATH=%PATH%;%SRCLKGDIR%

%_VECHO% Path = '%PATH%'

REM
REM NOTE: Skip checking for the "EagleShell.exe" tool along the PATH because we
REM       just found it in the LKG location.
REM
GOTO skip_eagleShell

:skip_lastKnownGood

REM ****************************************************************************
REM *********************** Verify Eagle Shell (in PATH) ***********************
REM ****************************************************************************

%__ECHO% EagleShell.exe /? > NUL 2>&1

IF ERRORLEVEL 1 (
  ECHO The "EagleShell.exe" tool appears to be missing.
  GOTO usage
)

%_AECHO% Using "EagleShell.exe" tool from the PATH...

:skip_eagleShell

REM ****************************************************************************
REM *********** Add Default (NetFx20) Build Output Directory to PATH ***********
REM ****************************************************************************
REM
REM NOTE: Always add the build output directory to the PATH because when we
REM       later execute the "EagleShell.exe" tool, we expect to get the patch
REM       level for the current release, not the last known good release.  Also,
REM       to make absolutely sure that we always get the current release version
REM       of the "EagleShell.exe" tool, we must prepend it to the PATH instead
REM       of appending it.
REM
SET SRCBINDIR=%~dp0\..\..\bin\%CONFIGURATION%\bin
SET SRCBINDIR=%SRCBINDIR:\\=\%

%_VECHO% SrcBinDir = '%SRCBINDIR%'

SET PATH=%SRCBINDIR%;%PATH%

%_VECHO% Path = '%PATH%'

REM ****************************************************************************
REM ***************** Verify Elevated Administrator Privileges *****************
REM ****************************************************************************
REM
REM NOTE: Using Eagle, make sure we have [elevated] administrator privileges
REM       before continuing unless we have been forbidden from doing so.
REM
IF DEFINED NOADMINISTRATOR GOTO skip_isAdministrator

SET IS_ADMINISTRATOR_CMD=EagleShell.exe -eval "puts stdout [isAdministrator]"

IF DEFINED __ECHO (
  %__ECHO% %IS_ADMINISTRATOR_CMD%
  SET ADMINISTRATOR=True
) ELSE (
  FOR /F %%T IN ('%IS_ADMINISTRATOR_CMD%') DO (SET ADMINISTRATOR=%%T)
)

IF NOT DEFINED ADMINISTRATOR (
  ECHO The ADMINISTRATOR environment variable could not be set.
  GOTO usage
)

%_VECHO% Administrator = '%ADMINISTRATOR%'

IF "%ADMINISTRATOR%" == "False" (
  ECHO This tool requires [elevated] administrator privileges.
  GOTO usage
)

:skip_isAdministrator

REM ****************************************************************************
REM ************************* Verify Release Directory *************************
REM ****************************************************************************

IF NOT EXIST "%ROOT%\Releases" (
  %__ECHO% MKDIR "%ROOT%\Releases"

  IF ERRORLEVEL 1 (
    ECHO Could not create directory "%ROOT%\Releases".
    GOTO errors
  )
)

REM ****************************************************************************
REM ************************ Clean Build/Release Output ************************
REM ****************************************************************************

IF NOT DEFINED NOCLEAN (
  REM
  REM HACK: The clean tool is being run from this tool; therefore, we must
  REM       have it skip trying to delete the log file that we are currently
  REM       using, if any.  The clean tool assumes that this tool uses log
  REM       file names matching the pattern "%TEMP%\EagleFlight*.log".  This
  REM       logging is typically accomplished by redirecting the output of
  REM       this tool on the command line.
  REM
  SET NOFLIGHT=1

  %__ECHO3% CALL "%TOOLS%\clean.bat"

  IF ERRORLEVEL 1 (
    ECHO Cleaning failed.
    GOTO errors
  )

  CALL :fn_UnsetVariable NOFLIGHT
)

REM ****************************************************************************
REM **************************** Set Build Suffixes ****************************
REM ****************************************************************************

SET DEFAULT_SUFFIX=NetFx20
SET NETFX40_SUFFIX=NetFx40
SET BARE_SUFFIX=Bare
SET LEAN_SUFFIX=LeanAndMean
SET DATABASE_SUFFIX=Database
SET UNIX_SUFFIX=MonoOnUnix

%_VECHO% DefaultSuffix = '%DEFAULT_SUFFIX%'
%_VECHO% NetFx40Suffix = '%NETFX40_SUFFIX%'
%_VECHO% BareSuffix = '%BARE_SUFFIX%'
%_VECHO% LeanSuffix = '%LEAN_SUFFIX%'
%_VECHO% DatabaseSuffix = '%DATABASE_SUFFIX%'
%_VECHO% UnixSuffix = '%UNIX_SUFFIX%'

REM ****************************************************************************
REM ************************* Set Build Configurations *************************
REM ****************************************************************************

IF NOT DEFINED DEFAULT_CONFIGURATION (
  %_AECHO% No "%DEFAULT_SUFFIX%" configuration specified, using default...
  SET DEFAULT_CONFIGURATION=%CONFIGURATION%
)

%_VECHO% DefaultConfiguration = '%DEFAULT_CONFIGURATION%'

IF NOT DEFINED NETFX40_CONFIGURATION (
  %_AECHO% No "%NETFX40_SUFFIX%" configuration specified, using default...

  IF NOT DEFINED NOPACKAGE (
    SET NETFX40_CONFIGURATION=%CONFIGURATION%All
  ) ELSE (
    SET NETFX40_CONFIGURATION=%CONFIGURATION%
  )
)

%_VECHO% NetFx40Configuration = '%NETFX40_CONFIGURATION%'

IF NOT DEFINED BARE_CONFIGURATION (
  %_AECHO% No "%BARE_SUFFIX%" configuration specified, using default...
  SET BARE_CONFIGURATION=%CONFIGURATION%
)

%_VECHO% BareConfiguration = '%BARE_CONFIGURATION%'

IF NOT DEFINED LEAN_CONFIGURATION (
  %_AECHO% No "%LEAN_SUFFIX%" configuration specified, using default...
  SET LEAN_CONFIGURATION=%CONFIGURATION%
)

%_VECHO% LeanConfiguration = '%LEAN_CONFIGURATION%'

IF NOT DEFINED DATABASE_CONFIGURATION (
  %_AECHO% No "%DATABASE_SUFFIX%" configuration specified, using default...
  SET DATABASE_CONFIGURATION=%CONFIGURATION%
)

%_VECHO% DatabaseConfiguration = '%DATABASE_CONFIGURATION%'

IF NOT DEFINED UNIX_CONFIGURATION (
  %_AECHO% No "%UNIX_SUFFIX%" configuration specified, using default...
  SET UNIX_CONFIGURATION=%CONFIGURATION%
)

%_VECHO% UnixConfiguration = '%UNIX_CONFIGURATION%'

REM ****************************************************************************
REM **************************** Skip Build Steps? *****************************
REM ****************************************************************************

IF DEFINED NOBUILD (
  %_AECHO% Skipping build steps...
  GOTO skip_build
)

REM ****************************************************************************
REM ****************** Default (NetFx20) Build Configuration *******************
REM ****************************************************************************

SET DEFAULT_FLAGS=%DEFAULT_FLAGS% /property:EagleBuildType=%DEFAULT_SUFFIX%
SET DEFAULT_FLAGS=%DEFAULT_FLAGS% /property:EagleTestType=%DEFAULT_SUFFIX%

IF NOT DEFINED NOTEST (
  SET DEFAULT_FLAGS=%DEFAULT_FLAGS% /property:EagleRunTests=true
)

IF NOT DEFINED NOPERFORMANCE (
  SET DEFAULT_FLAGS=%DEFAULT_FLAGS% /property:EagleTestPerformance=true
) ELSE (
  SET DEFAULT_FLAGS=%DEFAULT_FLAGS% /property:EagleTestPerformance=false
)

%_VECHO% DefaultFlags = '%DEFAULT_FLAGS%'

REM ****************************************************************************
REM *********************** NetFx40 Build Configuration ************************
REM ****************************************************************************

SET NETFX40_FLAGS=%NETFX40_FLAGS% /property:EagleBuildType=%NETFX40_SUFFIX%
SET NETFX40_FLAGS=%NETFX40_FLAGS% /property:EagleTestType=%NETFX40_SUFFIX%

IF NOT DEFINED NOTEST (
  SET NETFX40_FLAGS=%NETFX40_FLAGS% /property:EagleRunTests=true
)

IF NOT DEFINED NOTEST (
  SET NETFX40_FLAGS=%NETFX40_FLAGS% /property:EagleTestPerformance=false
)

%_VECHO% NetFx40Flags = '%NETFX40_FLAGS%'

REM ****************************************************************************
REM ************************* Bare Build Configuration *************************
REM ****************************************************************************

SET BARE_FLAGS=%BARE_FLAGS% /property:EagleBuildType=%BARE_SUFFIX%
SET BARE_FLAGS=%BARE_FLAGS% /property:EagleTestType=%BARE_SUFFIX%

IF NOT DEFINED NOTEST (
  SET BARE_FLAGS=%BARE_FLAGS% /property:EagleRunTests=true
)

IF NOT DEFINED NOTEST (
  SET BARE_FLAGS=%BARE_FLAGS% /property:EagleTestPerformance=false
)

%_VECHO% BareFlags = '%BARE_FLAGS%'

REM ****************************************************************************
REM ********************* LeanAndMean Build Configuration **********************
REM ****************************************************************************

SET LEAN_FLAGS=%LEAN_FLAGS% /property:EagleBuildType=%LEAN_SUFFIX%
SET LEAN_FLAGS=%LEAN_FLAGS% /property:EagleTestType=%LEAN_SUFFIX%

IF NOT DEFINED NOTEST (
  SET LEAN_FLAGS=%LEAN_FLAGS% /property:EagleRunTests=true
)

IF NOT DEFINED NOTEST (
  SET LEAN_FLAGS=%LEAN_FLAGS% /property:EagleTestPerformance=false
)

%_VECHO% LeanFlags = '%LEAN_FLAGS%'

REM ****************************************************************************
REM *********************** Database Build Configuration ***********************
REM ****************************************************************************

SET DATABASE_FLAGS=%DATABASE_FLAGS% /property:EagleBuildType=%DATABASE_SUFFIX%
SET DATABASE_FLAGS=%DATABASE_FLAGS% /property:EagleTestType=%DATABASE_SUFFIX%

IF NOT DEFINED NOTEST (
  SET DATABASE_FLAGS=%DATABASE_FLAGS% /property:EagleRunTests=true
)

IF NOT DEFINED NOTEST (
  SET DATABASE_FLAGS=%DATABASE_FLAGS% /property:EagleTestPerformance=false
)

%_VECHO% DatabaseFlags = '%DATABASE_FLAGS%'

REM ****************************************************************************
REM ********************** MonoOnUnix Build Configuration **********************
REM ****************************************************************************

SET UNIX_FLAGS=%UNIX_FLAGS% /property:EagleBuildType=%UNIX_SUFFIX%
SET UNIX_FLAGS=%UNIX_FLAGS% /property:EagleTestType=%UNIX_SUFFIX%

IF NOT DEFINED NOTEST (
  SET UNIX_FLAGS=%UNIX_FLAGS% /property:EagleRunTests=true
)

IF NOT DEFINED NOTEST (
  SET UNIX_FLAGS=%UNIX_FLAGS% /property:EagleTestPerformance=false
)

%_VECHO% UnixFlags = '%UNIX_FLAGS%'

REM ****************************************************************************
REM ************************** Set Build Release Info **************************
REM ****************************************************************************

IF NOT DEFINED NOATTRIBUTE (
  IF DEFINED RELEASE_ATTRIBUTE (
    FOR %%T IN (%TAGFILES%) DO (
      FOR /F "delims=" %%F IN ('DIR /B /S "%ROOT%%TAGPATTERN%\%%T" 2^> NUL') DO (
        %__ECHO% ECHO %RELEASE_ATTRIBUTE% %PIPE% EagleShell.exe -file "%TOOLS%\versionTag.eagle" AssemblyReleaseMode "%%F"

        IF ERRORLEVEL 1 (
          ECHO Updating "%%F" failed.
          GOTO errors
        )
      )
    )
  )
)

REM ****************************************************************************
REM ************************** Set Build Source Info ***************************
REM ****************************************************************************

IF NOT DEFINED NOSOURCEID (
  FOR %%T IN (%TAGFILES%) DO (
    FOR /F "delims=" %%F IN ('DIR /B /S "%ROOT%%TAGPATTERN%\%%T" 2^> NUL') DO (
      %__ECHO% EagleShell.exe -file "%TOOLS%\versionTag.eagle" SourceIdMode "%%F"

      IF ERRORLEVEL 1 (
        ECHO Updating "%%F" failed.
        GOTO errors
      )
    )
  )

  IF NOT DEFINED NOPACKAGE (
    FOR %%T IN (%PACKAGE_TAGFILES%) DO (
      FOR /F "delims=" %%F IN ('DIR /B /S "%ROOT%\Package\%%T" 2^> NUL') DO (
        %__ECHO% EagleShell.exe -file "%TOOLS%\versionTag.eagle" SourceIdMode "%%F"

        IF ERRORLEVEL 1 (
          ECHO Updating "%%F" failed.
          GOTO errors
        )
      )
    )
  )
)

REM ****************************************************************************
REM ************************* Build Default (NetFx20) **************************
REM ****************************************************************************

IF NOT DEFINED NODEFAULT (
  SET LOGSUFFIX=%DEFAULT_SUFFIX%
  CALL :fn_ShowVariable LogSuffix LOGSUFFIX

  IF NOT DEFINED NOFRAMEWORK (
    SET NETFX35ONLY=1
  )

  IF NOT DEFINED COMMERCIAL (
    SET NOCOMMERCIAL=1
  )

  IF NOT DEFINED ENTERPRISE (
    SET NOENTERPRISE=1
  )

  IF NOT DEFINED NOBUILDINFO (
    FOR %%T IN (%TAGFILES%) DO (
      FOR /F "delims=" %%F IN ('DIR /B /S "%ROOT%%TAGPATTERN%\%%T" 2^> NUL') DO (
        %__ECHO% ECHO %DEFAULT_SUFFIX% %PIPE% EagleShell.exe -file "%TOOLS%\versionTag.eagle" AssemblyTextMode "%%F"

        IF ERRORLEVEL 1 (
          ECHO Updating "%%F" failed.
          GOTO errors
        )
      )
    )
  )

  %__ECHO3% CALL "%TOOLS%\build.bat" "%DEFAULT_CONFIGURATION%" %DEFAULT_FLAGS% /property:FlightType=%ARGS%

  IF ERRORLEVEL 1 (
    ECHO Building "%CONFIGURATION%%DEFAULT_SUFFIX%" binaries failed.
    GOTO errors
  )

  IF NOT DEFINED ENTERPRISE (
    CALL :fn_UnsetVariable NOENTERPRISE
  )

  IF NOT DEFINED COMMERCIAL (
    CALL :fn_UnsetVariable NOCOMMERCIAL
  )

  IF NOT DEFINED NOFRAMEWORK (
    CALL :fn_UnsetVariable NETFX35ONLY
  )

  CALL :fn_UnsetVariable LOGSUFFIX
)

REM ****************************************************************************
REM *********************** Synchronize Build PatchLevel ***********************
REM ****************************************************************************

IF NOT DEFINED NOPATCHLEVEL (
  REM
  REM NOTE: All files named "PatchLevel.cs" must be updated with the correct
  REM       build number prior to building any non-default ^(NetFX20^) build
  REM       configuration.  If this is not done, building of the non-default
  REM       build configurations may fail because various projects may try
  REM       to include an AssemblyVersion attribute with a null value, due
  REM       to the EaglePatchLevel build property being true for those build
  REM       configurations.
  REM
  FOR %%T IN (%PATCHLEVELFILES%) DO (
    FOR /F "delims=" %%F IN ('DIR /B /S "%ROOT%%TAGPATTERN%\%%T" 2^> NUL') DO (
      %__ECHO% EagleShell.exe -file "%TOOLS%\versionTag.eagle" PatchLevelMode "%%F"

      IF ERRORLEVEL 1 (
        ECHO Updating "%%F" failed.
        GOTO errors
      )
    )
  )
)

REM ****************************************************************************
REM ****************************** Build NetFx40 *******************************
REM ****************************************************************************

IF NOT DEFINED NONETFX40 (
  SET LOGSUFFIX=%NETFX40_SUFFIX%
  CALL :fn_ShowVariable LogSuffix LOGSUFFIX

  IF NOT DEFINED NOFRAMEWORK (
    SET NETFX40ONLY=1
  )

  IF NOT DEFINED COMMERCIAL (
    SET NOCOMMERCIAL=1
  )

  IF NOT DEFINED ENTERPRISE (
    SET NOENTERPRISE=1
  )

  IF NOT DEFINED NOBUILDINFO (
    FOR %%T IN (%TAGFILES%) DO (
      FOR /F "delims=" %%F IN ('DIR /B /S "%ROOT%%TAGPATTERN%\%%T" 2^> NUL') DO (
        %__ECHO% ECHO %NETFX40_SUFFIX% %PIPE% EagleShell.exe -file "%TOOLS%\versionTag.eagle" AssemblyTextMode "%%F"

        IF ERRORLEVEL 1 (
          ECHO Updating "%%F" failed.
          GOTO errors
        )
      )
    )
  )

  %__ECHO3% CALL "%TOOLS%\build.bat" "%NETFX40_CONFIGURATION%" %NETFX40_FLAGS% /property:FlightType=%ARGS%

  IF ERRORLEVEL 1 (
    ECHO Building "%CONFIGURATION%%NETFX40_SUFFIX%" binaries failed.
    GOTO errors
  )

  IF NOT DEFINED ENTERPRISE (
    CALL :fn_UnsetVariable NOENTERPRISE
  )

  IF NOT DEFINED COMMERCIAL (
    CALL :fn_UnsetVariable NOCOMMERCIAL
  )

  IF NOT DEFINED NOFRAMEWORK (
    CALL :fn_UnsetVariable NETFX40ONLY
  )

  CALL :fn_UnsetVariable LOGSUFFIX
)

REM ****************************************************************************
REM ******************************** Build Bare ********************************
REM ****************************************************************************

IF NOT DEFINED NOBARE (
  SET LOGSUFFIX=%BARE_SUFFIX%
  CALL :fn_ShowVariable LogSuffix LOGSUFFIX

  IF NOT DEFINED NOFRAMEWORK (
    SET NETFX35ONLY=1
  )

  IF NOT DEFINED COMMERCIAL (
    SET NOCOMMERCIAL=1
  )

  IF NOT DEFINED ENTERPRISE (
    SET NOENTERPRISE=1
  )

  IF NOT DEFINED NOBUILDINFO (
    FOR %%T IN (%TAGFILES%) DO (
      FOR /F "delims=" %%F IN ('DIR /B /S "%ROOT%%TAGPATTERN%\%%T" 2^> NUL') DO (
        %__ECHO% ECHO %BARE_SUFFIX% %PIPE% EagleShell.exe -file "%TOOLS%\versionTag.eagle" AssemblyTextMode "%%F"

        IF ERRORLEVEL 1 (
          ECHO Updating "%%F" failed.
          GOTO errors
        )
      )
    )
  )

  %__ECHO3% CALL "%TOOLS%\build.bat" "%BARE_CONFIGURATION%" %BARE_FLAGS% /property:FlightType=%ARGS%

  IF ERRORLEVEL 1 (
    ECHO Building "%CONFIGURATION%%BARE_SUFFIX%" binaries failed.
    GOTO errors
  )

  IF NOT DEFINED ENTERPRISE (
    CALL :fn_UnsetVariable NOENTERPRISE
  )

  IF NOT DEFINED COMMERCIAL (
    CALL :fn_UnsetVariable NOCOMMERCIAL
  )

  IF NOT DEFINED NOFRAMEWORK (
    CALL :fn_UnsetVariable NETFX35ONLY
  )

  CALL :fn_UnsetVariable LOGSUFFIX
)

REM ****************************************************************************
REM **************************** Build LeanAndMean *****************************
REM ****************************************************************************

IF NOT DEFINED NOLEAN (
  SET LOGSUFFIX=%LEAN_SUFFIX%
  CALL :fn_ShowVariable LogSuffix LOGSUFFIX

  IF NOT DEFINED NOFRAMEWORK (
    SET NETFX35ONLY=1
  )

  IF NOT DEFINED COMMERCIAL (
    SET NOCOMMERCIAL=1
  )

  IF NOT DEFINED ENTERPRISE (
    SET NOENTERPRISE=1
  )

  IF NOT DEFINED NOBUILDINFO (
    FOR %%T IN (%TAGFILES%) DO (
      FOR /F "delims=" %%F IN ('DIR /B /S "%ROOT%%TAGPATTERN%\%%T" 2^> NUL') DO (
        %__ECHO% ECHO %LEAN_SUFFIX% %PIPE% EagleShell.exe -file "%TOOLS%\versionTag.eagle" AssemblyTextMode "%%F"

        IF ERRORLEVEL 1 (
          ECHO Updating "%%F" failed.
          GOTO errors
        )
      )
    )
  )

  %__ECHO3% CALL "%TOOLS%\build.bat" "%LEAN_CONFIGURATION%" %LEAN_FLAGS% /property:FlightType=%ARGS%

  IF ERRORLEVEL 1 (
    ECHO Building "%CONFIGURATION%%LEAN_SUFFIX%" binaries failed.
    GOTO errors
  )

  IF NOT DEFINED ENTERPRISE (
    CALL :fn_UnsetVariable NOENTERPRISE
  )

  IF NOT DEFINED COMMERCIAL (
    CALL :fn_UnsetVariable NOCOMMERCIAL
  )

  IF NOT DEFINED NOFRAMEWORK (
    CALL :fn_UnsetVariable NETFX35ONLY
  )

  CALL :fn_UnsetVariable LOGSUFFIX
)

REM ****************************************************************************
REM ****************************** Build Database ******************************
REM ****************************************************************************

IF NOT DEFINED NODATABASE (
  SET LOGSUFFIX=%DATABASE_SUFFIX%
  CALL :fn_ShowVariable LogSuffix LOGSUFFIX

  IF NOT DEFINED NOFRAMEWORK (
    SET NETFX35ONLY=1
  )

  IF NOT DEFINED COMMERCIAL (
    SET NOCOMMERCIAL=1
  )

  IF NOT DEFINED ENTERPRISE (
    SET NOENTERPRISE=1
  )

  IF NOT DEFINED NOBUILDINFO (
    FOR %%T IN (%TAGFILES%) DO (
      FOR /F "delims=" %%F IN ('DIR /B /S "%ROOT%%TAGPATTERN%\%%T" 2^> NUL') DO (
        %__ECHO% ECHO %DATABASE_SUFFIX% %PIPE% EagleShell.exe -file "%TOOLS%\versionTag.eagle" AssemblyTextMode "%%F"

        IF ERRORLEVEL 1 (
          ECHO Updating "%%F" failed.
          GOTO errors
        )
      )
    )
  )

  %__ECHO3% CALL "%TOOLS%\build.bat" "%DATABASE_CONFIGURATION%" %DATABASE_FLAGS% /property:FlightType=%ARGS%

  IF ERRORLEVEL 1 (
    ECHO Building "%CONFIGURATION%%DATABASE_SUFFIX%" binaries failed.
    GOTO errors
  )

  IF NOT DEFINED ENTERPRISE (
    CALL :fn_UnsetVariable NOENTERPRISE
  )

  IF NOT DEFINED COMMERCIAL (
    CALL :fn_UnsetVariable NOCOMMERCIAL
  )

  IF NOT DEFINED NOFRAMEWORK (
    CALL :fn_UnsetVariable NETFX35ONLY
  )

  CALL :fn_UnsetVariable LOGSUFFIX
)

REM ****************************************************************************
REM ***************************** Build MonoOnUnix *****************************
REM ****************************************************************************

IF NOT DEFINED NOUNIX (
  SET LOGSUFFIX=%UNIX_SUFFIX%
  CALL :fn_ShowVariable LogSuffix LOGSUFFIX

  IF NOT DEFINED NOFRAMEWORK (
    SET NETFX35ONLY=1
  )

  IF NOT DEFINED COMMERCIAL (
    SET NOCOMMERCIAL=1
  )

  IF NOT DEFINED ENTERPRISE (
    SET NOENTERPRISE=1
  )

  IF NOT DEFINED NOBUILDINFO (
    FOR %%T IN (%TAGFILES%) DO (
      FOR /F "delims=" %%F IN ('DIR /B /S "%ROOT%%TAGPATTERN%\%%T" 2^> NUL') DO (
        %__ECHO% ECHO %UNIX_SUFFIX% %PIPE% EagleShell.exe -file "%TOOLS%\versionTag.eagle" AssemblyTextMode "%%F"

        IF ERRORLEVEL 1 (
          ECHO Updating "%%F" failed.
          GOTO errors
        )
      )
    )
  )

  %__ECHO3% CALL "%TOOLS%\build.bat" "%UNIX_CONFIGURATION%" %UNIX_FLAGS% /property:FlightType=%ARGS%

  IF ERRORLEVEL 1 (
    ECHO Building "%CONFIGURATION%%UNIX_SUFFIX%" binaries failed.
    GOTO errors
  )

  IF NOT DEFINED ENTERPRISE (
    CALL :fn_UnsetVariable NOENTERPRISE
  )

  IF NOT DEFINED COMMERCIAL (
    CALL :fn_UnsetVariable NOCOMMERCIAL
  )

  IF NOT DEFINED NOFRAMEWORK (
    CALL :fn_UnsetVariable NETFX35ONLY
  )

  CALL :fn_UnsetVariable LOGSUFFIX
)

IF DEFINED BUILDONLY (
  %_AECHO% Build phase complete in build-only mode, exiting...
  GOTO no_errors
)

:skip_build

REM ****************************************************************************
REM ************************** Create Setup Packages ***************************
REM ****************************************************************************

IF NOT DEFINED NOBAKE (
  SET SKIP_SIGN_UNINSTALLER=1

  IF NOT DEFINED NODEFAULT (
    CALL :fn_UnsetVariable SUFFIX
    CALL :fn_ShowVariable Suffix SUFFIX

    %__ECHO3% CALL "%TOOLS%\bake.bat" "%CONFIGURATION%"

    IF ERRORLEVEL 1 (
      ECHO Baking "%CONFIGURATION%%DEFAULT_SUFFIX%" setup failed.
      GOTO errors
    )

    REM CALL :fn_UnsetVariable SUFFIX
  )

  IF NOT DEFINED NONETFX40 (
    SET SUFFIX=%NETFX40_SUFFIX%_
    CALL :fn_ShowVariable Suffix SUFFIX

    %__ECHO3% CALL "%TOOLS%\bake.bat" "%CONFIGURATION%%NETFX40_SUFFIX%"

    IF ERRORLEVEL 1 (
      ECHO Baking "%CONFIGURATION%%NETFX40_SUFFIX%" setup failed.
      GOTO errors
    )

    CALL :fn_UnsetVariable SUFFIX
  )

  IF NOT DEFINED NOPACKAGE (
    CALL :fn_UnsetVariable SUFFIX
    CALL :fn_ShowVariable Suffix SUFFIX

    %__ECHO3% CALL "%PACKAGE_TOOLS%\bake.bat" "%PACKAGE_PLATFORM%" "%PACKAGE_CONFIGURATION%"

    IF ERRORLEVEL 1 (
      ECHO Baking "%PACKAGE_PLATFORM%\%PACKAGE_CONFIGURATION%" setup failed.
      GOTO errors
    )

    REM CALL :fn_UnsetVariable SUFFIX
  )

  CALL :fn_UnsetVariable SKIP_SIGN_UNINSTALLER
)

REM ****************************************************************************
REM ********************* Tag ChangeLog/Update/NuGet Files *********************
REM ****************************************************************************

IF NOT DEFINED NOTAG (
  IF EXIST "%TOOLS%\..\..\ChangeLog" (
    %__ECHO% EagleShell.exe -file "%TOOLS%\versionTag.eagle" ChangeLogMode "%TOOLS%\..\..\ChangeLog"

    IF ERRORLEVEL 1 (
      ECHO Updating "%TOOLS%\..\..\ChangeLog" failed.
      GOTO errors
    )
  )

  IF EXIST "%TOOLS%\..\..\web\update.txt" (
    %__ECHO% EagleShell.exe -file "%TOOLS%\versionTag.eagle" UpdateMode "%TOOLS%\..\..\web\update.txt"

    IF ERRORLEVEL 1 (
      ECHO Updating "%TOOLS%\..\..\web\update.txt" failed.
      GOTO errors
    )
  )

  IF EXIST "%TOOLS%\..\..\Eagle.nuspec" (
    %__ECHO% EagleShell.exe -file "%TOOLS%\versionTag.eagle" NuSpecMode "%TOOLS%\..\..\Eagle.nuspec"

    IF ERRORLEVEL 1 (
      ECHO Updating "%TOOLS%\..\..\Eagle.nuspec" failed.
      GOTO errors
    )
  )

  IF EXIST "%TOOLS%\..\..\Eagle.src.nuspec" (
    %__ECHO% EagleShell.exe -file "%TOOLS%\versionTag.eagle" NuSpecMode "%TOOLS%\..\..\Eagle.src.nuspec"

    IF ERRORLEVEL 1 (
      ECHO Updating "%TOOLS%\..\..\Eagle.src.nuspec" failed.
      GOTO errors
    )
  )
)

REM ****************************************************************************
REM ************************** Query Eagle PatchLevel **************************
REM ****************************************************************************

SET GET_PATCHLEVEL_CMD=EagleShell.exe -eval "puts stdout [info engine PatchLevel]"

IF DEFINED __ECHO (
  %__ECHO% %GET_PATCHLEVEL_CMD%
  SET PATCHLEVEL=1.0.X.X
) ELSE (
  FOR /F %%T IN ('%GET_PATCHLEVEL_CMD%') DO (SET PATCHLEVEL=%%T)
)

IF NOT DEFINED PATCHLEVEL (
  ECHO The PATCHLEVEL environment variable could not be set.
  GOTO errors
)

%_VECHO% PatchLevel = '%PATCHLEVEL%'

REM ****************************************************************************
REM ************************* Query Garuda PatchLevel **************************
REM ****************************************************************************

SET GET_PACKAGE_PATCHLEVEL_CMD=EagleShell.exe -eval "puts stdout [file version {%ROOT%\bin\%PACKAGE_PLATFORM%\%PACKAGE_CONFIGURATION%\Garuda.dll}]"

IF DEFINED __ECHO (
  %__ECHO% %GET_PACKAGE_PATCHLEVEL_CMD%
  SET PACKAGE_PATCHLEVEL=1.0.X.X
) ELSE (
  FOR /F %%T IN ('%GET_PACKAGE_PATCHLEVEL_CMD%') DO (SET PACKAGE_PATCHLEVEL=%%T)
)

IF NOT DEFINED PACKAGE_PATCHLEVEL (
  ECHO The PACKAGE_PATCHLEVEL environment variable could not be set.
  GOTO errors
)

%_VECHO% PackagePatchLevel = '%PACKAGE_PATCHLEVEL%'

REM ****************************************************************************
REM ************************** Create Source Package ***************************
REM ****************************************************************************

IF NOT DEFINED NOARCHIVE (
  %__ECHO3% CALL "%TOOLS%\archive.bat" "%CONFIGURATION%"

  IF ERRORLEVEL 1 (
    ECHO Creating source archives failed.
    GOTO errors
  )
)

REM ****************************************************************************
REM ************************** Create Binary Packages **************************
REM ****************************************************************************

IF NOT DEFINED NORELEASE (
  IF NOT DEFINED NODEFAULT (
    CALL :fn_UnsetVariable SUFFIX
    CALL :fn_ShowVariable Suffix SUFFIX

    IF NOT DEFINED NOBINARY (
      %__ECHO3% CALL "%TOOLS%\release.bat" "%CONFIGURATION%"

      IF ERRORLEVEL 1 (
        ECHO Creating "%CONFIGURATION%%DEFAULT_SUFFIX%" binaries archive failed.
        GOTO errors
      )
    )

    IF NOT DEFINED NORUNTIME (
      %__ECHO3% CALL "%TOOLS%\release.bat" "%CONFIGURATION%" Runtime exclude_runtime.txt

      IF ERRORLEVEL 1 (
        ECHO Creating "%CONFIGURATION%%DEFAULT_SUFFIX%" runtime binaries archive failed.
        GOTO errors
      )
    )

    IF NOT DEFINED NOCORE (
      %__ECHO3% CALL "%TOOLS%\release.bat" "%CONFIGURATION%" Core exclude_core.txt

      IF ERRORLEVEL 1 (
        ECHO Creating "%CONFIGURATION%%DEFAULT_SUFFIX%" core binaries archive failed.
        GOTO errors
      )
    )

    REM CALL :fn_UnsetVariable SUFFIX
  )

  IF NOT DEFINED NONETFX40 (
    SET SUFFIX=%NETFX40_SUFFIX%_
    CALL :fn_ShowVariable Suffix SUFFIX

    IF NOT DEFINED NOBINARY (
      %__ECHO3% CALL "%TOOLS%\release.bat" "%CONFIGURATION%%NETFX40_SUFFIX%"

      IF ERRORLEVEL 1 (
        ECHO Creating "%CONFIGURATION%%NETFX40_SUFFIX%" binaries archive failed.
        GOTO errors
      )
    )

    IF NOT DEFINED NORUNTIME (
      %__ECHO3% CALL "%TOOLS%\release.bat" "%CONFIGURATION%%NETFX40_SUFFIX%" Runtime exclude_runtime.txt

      IF ERRORLEVEL 1 (
        ECHO Creating "%CONFIGURATION%%NETFX40_SUFFIX%" runtime binaries archive failed.
        GOTO errors
      )
    )

    IF NOT DEFINED NOCORE (
      %__ECHO3% CALL "%TOOLS%\release.bat" "%CONFIGURATION%%NETFX40_SUFFIX%" Core exclude_core.txt

      IF ERRORLEVEL 1 (
        ECHO Creating "%CONFIGURATION%%NETFX40_SUFFIX%" core binaries archive failed.
        GOTO errors
      )
    )

    CALL :fn_UnsetVariable SUFFIX
  )

  IF NOT DEFINED NOBARE (
    SET SUFFIX=%BARE_SUFFIX%
    CALL :fn_ShowVariable Suffix SUFFIX

    IF NOT DEFINED NOBINARY (
      %__ECHO3% CALL "%TOOLS%\release.bat" "%CONFIGURATION%%BARE_SUFFIX%"

      IF ERRORLEVEL 1 (
        ECHO Creating "%CONFIGURATION%%BARE_SUFFIX%" binaries archive failed.
        GOTO errors
      )
    )

    IF NOT DEFINED NORUNTIME (
      %__ECHO3% CALL "%TOOLS%\release.bat" "%CONFIGURATION%%BARE_SUFFIX%" Runtime exclude_runtime.txt

      IF ERRORLEVEL 1 (
        ECHO Creating "%CONFIGURATION%%BARE_SUFFIX%" runtime binaries archive failed.
        GOTO errors
      )
    )

    IF NOT DEFINED NOCORE (
      %__ECHO3% CALL "%TOOLS%\release.bat" "%CONFIGURATION%%BARE_SUFFIX%" Core exclude_core.txt

      IF ERRORLEVEL 1 (
        ECHO Creating "%CONFIGURATION%%BARE_SUFFIX%" core binaries archive failed.
        GOTO errors
      )
    )

    CALL :fn_UnsetVariable SUFFIX
  )

  SET NOLIB=1

  IF NOT DEFINED NOLEAN (
    SET SUFFIX=%LEAN_SUFFIX%
    CALL :fn_ShowVariable Suffix SUFFIX

    IF NOT DEFINED NOBINARY (
      %__ECHO3% CALL "%TOOLS%\release.bat" "%CONFIGURATION%%LEAN_SUFFIX%"

      IF ERRORLEVEL 1 (
        ECHO Creating "%CONFIGURATION%%LEAN_SUFFIX%" binaries archive failed.
        GOTO errors
      )
    )

    IF NOT DEFINED NORUNTIME (
      %__ECHO3% CALL "%TOOLS%\release.bat" "%CONFIGURATION%%LEAN_SUFFIX%" Runtime exclude_runtime.txt

      IF ERRORLEVEL 1 (
        ECHO Creating "%CONFIGURATION%%LEAN_SUFFIX%" runtime binaries archive failed.
        GOTO errors
      )
    )

    IF NOT DEFINED NOCORE (
      %__ECHO3% CALL "%TOOLS%\release.bat" "%CONFIGURATION%%LEAN_SUFFIX%" Core exclude_core.txt

      IF ERRORLEVEL 1 (
        ECHO Creating "%CONFIGURATION%%LEAN_SUFFIX%" core binaries archive failed.
        GOTO errors
      )
    )

    CALL :fn_UnsetVariable SUFFIX
  )

  IF NOT DEFINED NODATABASE (
    SET SUFFIX=%DATABASE_SUFFIX%
    CALL :fn_ShowVariable Suffix SUFFIX

    IF NOT DEFINED NOBINARY (
      %__ECHO3% CALL "%TOOLS%\release.bat" "%CONFIGURATION%%DATABASE_SUFFIX%"

      IF ERRORLEVEL 1 (
        ECHO Creating "%CONFIGURATION%%DATABASE_SUFFIX%" binaries archive failed.
        GOTO errors
      )
    )

    IF NOT DEFINED NORUNTIME (
      %__ECHO3% CALL "%TOOLS%\release.bat" "%CONFIGURATION%%DATABASE_SUFFIX%" Runtime exclude_runtime.txt

      IF ERRORLEVEL 1 (
        ECHO Creating "%CONFIGURATION%%DATABASE_SUFFIX%" runtime binaries archive failed.
        GOTO errors
      )
    )

    IF NOT DEFINED NOCORE (
      %__ECHO3% CALL "%TOOLS%\release.bat" "%CONFIGURATION%%DATABASE_SUFFIX%" Core exclude_core.txt

      IF ERRORLEVEL 1 (
        ECHO Creating "%CONFIGURATION%%DATABASE_SUFFIX%" core binaries archive failed.
        GOTO errors
      )
    )

    CALL :fn_UnsetVariable SUFFIX
  )

  CALL :fn_UnsetVariable NOLIB

  IF NOT DEFINED NOUNIX (
    SET SUFFIX=%UNIX_SUFFIX%
    CALL :fn_ShowVariable Suffix SUFFIX

    IF NOT DEFINED NOBINARY (
      %__ECHO3% CALL "%TOOLS%\release.bat" "%CONFIGURATION%%UNIX_SUFFIX%"

      IF ERRORLEVEL 1 (
        ECHO Creating "%CONFIGURATION%%UNIX_SUFFIX%" binaries archive failed.
        GOTO errors
      )
    )

    IF NOT DEFINED NORUNTIME (
      %__ECHO3% CALL "%TOOLS%\release.bat" "%CONFIGURATION%%UNIX_SUFFIX%" Runtime exclude_runtime.txt

      IF ERRORLEVEL 1 (
        ECHO Creating "%CONFIGURATION%%UNIX_SUFFIX%" runtime binaries archive failed.
        GOTO errors
      )
    )

    IF NOT DEFINED NOCORE (
      %__ECHO3% CALL "%TOOLS%\release.bat" "%CONFIGURATION%%UNIX_SUFFIX%" Core exclude_core.txt

      IF ERRORLEVEL 1 (
        ECHO Creating "%CONFIGURATION%%UNIX_SUFFIX%" core binaries archive failed.
        GOTO errors
      )
    )

    CALL :fn_UnsetVariable SUFFIX
  )

  IF NOT DEFINED NOPACKAGE (
    CALL :fn_UnsetVariable SUFFIX
    CALL :fn_ShowVariable Suffix SUFFIX

    IF NOT DEFINED NOBINARY (
      %__ECHO3% CALL "%PACKAGE_TOOLS%\release.bat" "%PACKAGE_PLATFORM%" "%PACKAGE_CONFIGURATION%"

      IF ERRORLEVEL 1 (
        ECHO Creating "%PACKAGE_PLATFORM%\%PACKAGE_CONFIGURATION%" binaries archive failed.
        GOTO errors
      )
    )

    IF NOT DEFINED NORUNTIME (
      %__ECHO3% CALL "%PACKAGE_TOOLS%\release.bat" "%PACKAGE_PLATFORM%" "%PACKAGE_CONFIGURATION%" Runtime exclude_runtime.txt

      IF ERRORLEVEL 1 (
        ECHO Creating "%PACKAGE_PLATFORM%\%PACKAGE_CONFIGURATION%" runtime binaries archive failed.
        GOTO errors
      )
    )

    IF NOT DEFINED NOCORE (
      %__ECHO3% CALL "%PACKAGE_TOOLS%\release.bat" "%PACKAGE_PLATFORM%" "%PACKAGE_CONFIGURATION%" Core exclude_core.txt

      IF ERRORLEVEL 1 (
        ECHO Creating "%PACKAGE_PLATFORM%\%PACKAGE_CONFIGURATION%" core binaries archive failed.
        GOTO errors
      )
    )

    REM CALL :fn_UnsetVariable SUFFIX
  )
)

REM ****************************************************************************
REM ********************** Verify Release Archives Files ***********************
REM ****************************************************************************

IF NOT DEFINED NOVERIFY (
  %__ECHO% EagleShell.exe -file "%TOOLS%\verify.eagle" "%RELEASES%"

  IF ERRORLEVEL 1 (
    ECHO Verifying release archive files in "%RELEASES%" failed.
    GOTO errors
  )
)

REM ****************************************************************************
REM ************************ Copy Build/Test Log Files *************************
REM ****************************************************************************

IF NOT DEFINED NOLOGS (
  IF EXIST "%TEMP%\EagleShell.exe.test.*.log" (
    %__ECHO% XCOPY "%TEMP%\EagleShell.exe.test.*.log" "%RELEASES%" %FFLAGS% %DFLAGS%

    IF ERRORLEVEL 1 (
      ECHO Failed to copy "%TEMP%\EagleShell.exe.test.*.log" to "%RELEASES%".
      GOTO errors
    )
  )

  IF NOT DEFINED NOPACKAGE (
    IF EXIST "%TEMP%\Garuda*Build.*" (
      %__ECHO% XCOPY "%TEMP%\Garuda*Build.*" "%RELEASES%" %FFLAGS% %DFLAGS%

      IF ERRORLEVEL 1 (
        ECHO Failed to copy "%TEMP%\Garuda*Build.*" to "%RELEASES%".
        GOTO errors
      )
    )
  )
)

REM ****************************************************************************
REM ************************** Create NuGet Packages ***************************
REM ****************************************************************************

IF NOT DEFINED NONUGET (
  IF NOT DEFINED NODEFAULT (
    IF NOT EXIST "%NUGETBASEPATH%" (
      %__ECHO% MKDIR "%NUGETBASEPATH%"

      IF ERRORLEVEL 1 (
        ECHO Could not create directory "%NUGETBASEPATH%".
        GOTO errors
      )
    )

    IF NOT EXIST "%NUGETOUTPUTDIR%\NuGet" (
      %__ECHO% MKDIR "%NUGETOUTPUTDIR%\NuGet"

      IF ERRORLEVEL 1 (
        ECHO Could not create directory "%NUGETOUTPUTDIR%\NuGet".
        GOTO errors
      )
    )

    IF NOT EXIST "%NUGETOUTPUTDIR%\SymbolSource" (
      %__ECHO% MKDIR "%NUGETOUTPUTDIR%\SymbolSource"

      IF ERRORLEVEL 1 (
        ECHO Could not create directory "%NUGETOUTPUTDIR%\SymbolSource".
        GOTO errors
      )
    )

    %__ECHO% XCOPY "%SRCBINDIR%\Eagle.dll" "%NUGETBASEPATH%\bin\" %FFLAGS% %DFLAGS%

    IF ERRORLEVEL 1 (
      ECHO Failed to copy "%SRCBINDIR%\Eagle.dll" to "%NUGETBASEPATH%\bin\".
      GOTO errors
    )

    %__ECHO% XCOPY "%SRCBINDIR%\Eagle.pdb" "%NUGETBASEPATH%\bin\" %FFLAGS% %DFLAGS%

    IF ERRORLEVEL 1 (
      ECHO Failed to copy "%SRCBINDIR%\Eagle.pdb" to "%NUGETBASEPATH%\bin\".
      GOTO errors
    )

    REM
    REM HACK: We must change the current directory because the EXCLUDE argument
    REM       to the XCOPY command is very picky.  The first problem is that it
    REM       apparently cannot include surrounding double quotes, even though
    REM       this fact is not documented anywhere, thus making it impossible to
    REM       use any path that contains spaces.  The second problem is that all
    REM       the files we want to copy would match the exclusion rule "\Tools\"
    REM       because we are running from the "Tools" directory and we would
    REM       have to use our directory as the basis for the library source
    REM       directory ^(unless we could find a reasonable way of resolving it
    REM       to an absolute path beforehand^).
    REM
    %__ECHO2% PUSHD "%ROOT%"

    IF ERRORLEVEL 1 (
      ECHO Could not change directory to "%ROOT%".
      GOTO errors
    )

    REM
    REM HACK: Must use relative path for source files and EXCLUDE file here, see
    REM       above comment.
    REM
    %__ECHO% XCOPY "Library\*.cs" "%NUGETBASEPATH%\src\Library\" %FFLAGS% %DFLAGS% /S /EXCLUDE:exclude_nuget.txt

    IF ERRORLEVEL 1 (
      ECHO Failed to copy "Library\*.cs" to "%NUGETBASEPATH%\src\Library\".
      GOTO errors
    )

    %__ECHO2% POPD

    IF ERRORLEVEL 1 (
      ECHO Could not restore directory.
      GOTO errors
    )

    %__ECHO% NuGet.exe pack "%ROOT%\Eagle.nuspec" /BasePath "%NUGETBASEPATH%" /OutputDirectory "%NUGETOUTPUTDIR%\NuGet"

    IF ERRORLEVEL 1 (
      ECHO Failed to create NuGet package from "%ROOT%\Eagle.nuspec".
      GOTO errors
    )

    %__ECHO% NuGet.exe pack "%ROOT%\Eagle.src.nuspec" /BasePath "%NUGETBASEPATH%" /OutputDirectory "%NUGETOUTPUTDIR%\SymbolSource"

    IF ERRORLEVEL 1 (
      ECHO Failed to create SymbolSource package from "%ROOT%\Eagle.src.nuspec".
      GOTO errors
    )

    IF DEFINED NONUGETPUSH (
      %_AECHO% Skipping push of NuGet and SymbolSource packages...
      GOTO skip_nuGetPush
    )

    REM
    REM NOTE: Emit the necessary commands to the console to push the created
    REM       NuGet and SymbolSource packages; however, do not actually push the
    REM       created packages.  Eventually, we may decide to migrate to
    REM       automatically pushing the created packages; however, this will
    REM       have to be done with great care because we cannot really undo it.
    REM
    IF NOT DEFINED NUGET_API_KEY (
      ECHO The NUGET_API_KEY environment variable must be set first.
      GOTO usage
    )

    IF NOT DEFINED SYMBOLSOURCE_API_KEY (
      ECHO The SYMBOLSOURCE_API_KEY environment variable must be set first.
      GOTO usage
    )

    ECHO NuGet.exe push "%NUGETOUTPUTDIR%\NuGet\Eagle.%PATCHLEVEL%.nupkg" "%NUGET_API_KEY%"

    IF ERRORLEVEL 1 (
      ECHO Failed to push NuGet package "%NUGETOUTPUTDIR%\NuGet\Eagle.%PATCHLEVEL%.nupkg".
      GOTO errors
    )

    ECHO NuGet.exe push "%NUGETOUTPUTDIR%\SymbolSource\Eagle.%PATCHLEVEL%.nupkg" "%NUGET_API_KEY%" -source http://nuget.gw.symbolsource.org/Public/NuGet

    IF ERRORLEVEL 1 (
      ECHO Failed to push SymbolSource package "%NUGETOUTPUTDIR%\SymbolSource\Eagle.%PATCHLEVEL%.nupkg".
      GOTO errors
    )
  )
)

:skip_nuGetPush

REM ****************************************************************************
REM **************************** Hash All Packages *****************************
REM ****************************************************************************

IF NOT DEFINED NOHASH (
  IF EXIST "%TOOLS%\sha1sum.exe" (
    IF EXIST "%RELEASES%\hash.txt" (
      %__ECHO% DEL "%RELEASES%\hash.txt"

      IF ERRORLEVEL 1 (
        ECHO Could not delete file "%RELEASES%\hash.txt".
        GOTO errors
      )
    )

    %__ECHO% ECHO Hashed "%DATE%" at "%TIME%" by "%USERDOMAIN%\%USERNAME%" %APPEND% "%RELEASES%\hash.txt"
    %__ECHO% ECHO. %APPEND% "%RELEASES%\hash.txt"

    FOR %%B IN (%HASHBASES%) DO (
      FOR %%E IN (%HASHEXTS%) DO (
        FOR /F "delims=" %%F IN ('DIR /B /S /ON "%RELEASES%\%%B*%PATCHLEVEL%.%%E" 2^> NUL') DO (
          %_AECHO% Hashing file "%%F"...
          %__ECHO% "%TOOLS%\sha1sum.exe" "%%F" %APPEND% "%RELEASES%\hash.txt"

          IF ERRORLEVEL 1 (
            ECHO Hashing file "%%F" failed.
            GOTO errors
          )
        )

        FOR /F "delims=" %%F IN ('DIR /B /S /ON "%RELEASES%\%%B*%PACKAGE_PATCHLEVEL%.%%E" 2^> NUL') DO (
          %_AECHO% Hashing file "%%F"...
          %__ECHO% "%TOOLS%\sha1sum.exe" "%%F" %APPEND% "%RELEASES%\hash.txt"

          IF ERRORLEVEL 1 (
            ECHO Hashing file "%%F" failed.
            GOTO errors
          )
        )
      )
    )
  ) ELSE (
    %_AECHO% Hashing tool "%TOOLS%\sha1sum.exe" not found, skipping...
  )
)

GOTO no_errors

:fn_UnquoteVariable
  SETLOCAL
  IF NOT DEFINED %1 GOTO :EOF
  SET __ECHO_CMD=ECHO %%%1%%
  FOR /F "delims=" %%V IN ('%__ECHO_CMD%') DO (
    SET VALUE=%%V
  )
  SET VALUE=%VALUE:"=%
  REM "
  ENDLOCAL && SET %1=%VALUE%
  GOTO :EOF

:fn_ShowVariable
  SETLOCAL
  SET __ECHO_CMD=ECHO %%%2%%
  FOR /F "delims=" %%V IN ('%__ECHO_CMD%') DO (
    IF NOT "%%V" == "" (
      IF NOT "%%V" == "%%%2%%" (
        %_VECHO% %1 = '%%V'
      )
    )
  )
  ENDLOCAL
  GOTO :EOF

:fn_AppendVariable
  SET __ECHO_CMD=ECHO %%%1%%
  IF DEFINED %1 (
    FOR /F "delims=" %%V IN ('%__ECHO_CMD%') DO (
      SET %1=%%V%~2
    )
  ) ELSE (
    SET %1=%~2
  )
  SET __ECHO_CMD=
  CALL :fn_ResetErrorLevel
  GOTO :EOF

:fn_UnsetVariable
  IF NOT "%1" == "" (
    SET %1=
    CALL :fn_ResetErrorLevel
  )
  GOTO :EOF

:fn_ResetErrorLevel
  VERIFY > NUL
  GOTO :EOF

:fn_SetErrorLevel
  VERIFY MAYBE 2> NUL
  GOTO :EOF

:usage
  ECHO.
  ECHO Usage: %~nx0 [platform] [configuration] [...]
  ECHO.
  ECHO The NUGET_API_KEY environment variable must be set to the NuGet API key.
  ECHO.
  ECHO The SYMBOLSOURCE_API_KEY environment variable must be set to the SymbolSource
  ECHO API key.
  ECHO.
  ECHO The latest Eagle binaries ^(including the "EagleShell.exe" tool^) are required to
  ECHO exist in the "%%LKG%%\Eagle\bin" directory or somewhere along your PATH.  They
  ECHO can be downloaded for free from:
  ECHO.
  ECHO                          http://eagle.to/
  ECHO.
  ECHO Several of the tasks performed by this tool require [elevated] administrator
  ECHO privileges.  Therefore, this tool will refuse to run without them.
  GOTO errors

:errors
  CALL :fn_SetErrorLevel
  ENDLOCAL
  ECHO.
  ECHO Failure, errors were encountered.
  GOTO end_of_file

:no_errors
  CALL :fn_ResetErrorLevel
  ENDLOCAL
  ECHO.
  ECHO Success, no errors were encountered.
  GOTO end_of_file

:end_of_file
ECHO STOPPED ON %DATE% AT %TIME% BY %USERDOMAIN%\%USERNAME%
%__ECHO% EXIT /B %ERRORLEVEL%
