/*
 * FormatOps.cs --
 *
 * Copyright (c) 2007-2012 by Joe Mistachkin.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * RCS: @(#) $Id: $
 */

using System;
using System.Collections.Generic;
using System.Globalization;
using System.Reflection;
using System.Runtime.InteropServices;
using System.Security.Cryptography.X509Certificates;
using System.Text;

namespace Eagle._Components.Private
{
    [Guid("9fe47978-cf5c-43a7-8333-2402bb6649ee")]
    internal static class FormatOps
    {
        #region Private Constants
        internal const string NameAndValueFormat = "{0}: {1}";
        #endregion

        ///////////////////////////////////////////////////////////////////////

        #region String Formatting Methods
        public static string ForDisplay(
            object value
            )
        {
            if (value == null)
                return "<null>";

            string result = value.ToString();
            Type type = value.GetType();

            if (result.Length == 0)
                return "<empty>";

            result = String.Format(
                type.IsSubclassOf(typeof(ValueType)) ? "{0}" : "\"{0}\"",
                result);

            return result;
        }

        ///////////////////////////////////////////////////////////////////////

        public static string ToHexString(
            byte[] array
            )
        {
            if (array == null)
                return null;

            StringBuilder result = new StringBuilder();

            int length = array.Length;

            for (int index = 0; index < length; index++)
                result.AppendFormat("{0:x2}", array[index]);

            return result.ToString();
        }

        ///////////////////////////////////////////////////////////////////////

        public static string CertificateToString(
            X509Certificate2 certificate2,
            bool verbose
            )
        {
            if (certificate2 != null)
                return verbose ? certificate2.Subject :
                    certificate2.GetNameInfo(X509NameType.SimpleName, false);

            return null;
        }

        ///////////////////////////////////////////////////////////////////////

        public static string CultureToString(
            CultureInfo cultureInfo
            )
        {
            if (cultureInfo != null)
                return cultureInfo.DisplayName;

            return null;
        }

        ///////////////////////////////////////////////////////////////////////

        public static string DelegateToString(
            Delegate @delegate
            )
        {
            MethodInfo methodInfo = (@delegate != null) ? @delegate.Method : null;

            if (methodInfo != null)
                return String.Format("{0}{1}{2}", methodInfo.DeclaringType,
                    Type.Delimiter, methodInfo.Name);

            return null;
        }

        ///////////////////////////////////////////////////////////////////////

        public static string ListToString(
            IEnumerable<string> collection
            )
        {
            StringBuilder result = new StringBuilder();

            if (collection != null)
            {
                foreach (string item in collection)
                {
                    if (result.Length > 0)
                        result.Append(Characters.Space);

                    result.Append(item);
                }
            }

            return result.ToString();
        }

        ///////////////////////////////////////////////////////////////////////

        public static string NotesToString(
            string text
            )
        {
            StringBuilder result = new StringBuilder(text);

            result.Replace(Characters.Ampersand.ToString(),
                Characters.EscapedAmpersand);

            result.Replace(Characters.HorizontalTab.ToString(),
                Characters.EscapedHorizontalTab);

            result.Replace(Characters.VerticalTab.ToString(),
                Characters.EscapedVerticalTab);

            result.Replace(Characters.LineFeed.ToString(),
                Characters.EscapedLineFeed);

            result.Replace(Characters.CarriageReturn.ToString(),
                Characters.EscapedCarriageReturn);

            return result.ToString();
        }
        #endregion
    }
}
